<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kgraphviewer_part" version="4" translationDomain="kgraphviewer">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_export" group="file_merge"/>
    <Separator group="file_merge"/>
    <Action name="file_print" group="file_merge"/>
    <Action name="file_print_preview" group="file_merge"/>
    <Action name="file_page_setup" group="file_merge"/>
  </Menu>
  <Menu name="view"><text>&amp;View</text>
    <Action name="view_zoom_in"/>
    <Action name="view_zoom_out"/>
    <Separator/>
    <Action name="view_redisplay"/>
    <Separator/>
    <Action name="view_layout_algo"/>
    <Separator/>
    <Action name="view_bev_enabled"/>
    <Action name="view_bev"/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar">
  <!--Action name="file_print"/-->
  <Action name="file_print_preview"/>
  <!--Action name="file_page_setup"/-->
</ToolBar>
<ToolBar name="viewToolBar">
  <!--Action name="view_redisplay"/-->
  <!--Separator/-->
  <Action name="view_layout_algo"/>
  <Separator/>
  <Action name="view_zoom_in"/>
  <Action name="view_zoom_out"/>
  <Separator/>
  <Action name="view_bev_enabled"/>
  <!--Action name="view_bev"/-->
</ToolBar>
</gui>
