<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kgrapheditor" version="6" translationDomain="kgraphviewer">
<MenuBar>
  <Menu name="file">
    <DefineGroup name="file_merge" />
  </Menu>
  <Menu name="edit">
    <Action name="edit_new_vertex"/>
    <Action name="edit_new_edge"/>
  </Menu>
  <Menu name="view">
    <DefineGroup name="view_merge"/>
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar">
  <Action name="file_open"/>
  <Action name="file_save"/>
  <Action name="file_save_as"/>
<Merge/>
</ToolBar>
<ToolBar name="viewToolBar"><text>View Toolbar</text>
  <Merge/>
</ToolBar>
<ToolBar name="helpToolBar" align="right"><text>Help Toolbar</text>
  <Action name="help_whats_this"/>
  <Action name="help"/>
  <Merge/>
</ToolBar>
<ToolBar name="editToolBar"><text>Edit Toolbar</text>
  <Action name="edit_new_vertex"/>
  <Action name="edit_new_edge"/>
  <Merge/>
<Merge/>
</ToolBar>
</gui>
