<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="kgraphviewer" version="6" translationDomain="kgraphviewer">
<MenuBar>
  <Menu name="file">
    <DefineGroup name="file_merge"/>
  </Menu>
  <Menu name="view">
    <DefineGroup name="view_merge"/>
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar">
  <Action name="file_open"/>
  <Merge/>
</ToolBar>
<ToolBar name="viewToolBar"><text>View Toolbar</text>
  <Merge/>
</ToolBar>
<ToolBar name="helpToolBar" align="right"><text>Help Toolbar</text>
  <Action name="help_whats_this"/>
  <Action name="help"/>
  <Merge/>
</ToolBar>
</gui>
